﻿<# 
.SYNOPSIS
	Run as a scheduled task, grants Lync policy to members of an Active Directory security group

.DESCRIPTION
	Run as a scheduled task, grants Lync policy to members of an Active Directory security group. The name of the group, and the type and name of the policy to be granted are all customizable.

.NOTES
  Version    	      	: v1.6 - See changelog at http://www.ehloworld.com/1346 
  Wish list						: 
  Rights Required			: local admin on server it's running on; CsUserAdministrator or CsAdministrator in Lync
  Sched Task Required	: Yes - install mode will automatically create scheduled task to run every 4 hours
  Lync Version				: 2010
	Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Post			: http://www.ehloworld.com/1348
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
  											provided AS IS without warranty of any kind. I disclaim all implied warranties including, 
  											without limitation, any implied warranties of merchantability or of fitness for a particular
  											purpose. The entire risk arising out of the use or performance of the sample scripts and 
  											documentation remains with you. In no event shall I be liable for any damages whatsoever 
  											(including, without limitation, damages for loss of business profits, business interruption,
  											loss of business information, or other pecuniary loss) arising out of the use of or inability
  											to use the script or documentation. 
  Acknowledgements 		: http://jeffwouters.nl/index.php/2012/06/powershell-function-to-get-all-nested-group-members-in-active-directory/
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					: 
  Known issues				: None yet, but I'm sure you'll find some!   
    
.LINK     
	http://www.ehloworld.com/1348

.EXAMPLE 
	.\Grant-CsPolicyByADGroup.ps1
	
	Description
	-----------
	Looks at members of a defined AD group, and grants a Lync policy to each

.EXAMPLE 
	.\Grant-CsPolicyByADGroup.ps1 -install
	
	Description
	-----------
	Creates the scheduled task for the script to run everyday at 6am. It will prompt for the password for the currently logged on user.

.INPUTS
	None. You cannot pipe objects to this script
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True, DefaultParameterSetName = "Default")]
param(
	# DN of group to check for membership
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, ParameterSetName = "Default")]
	[ValidateNotNullOrEmpty()]
	[string] $GroupDN,
	
	# Name of policy to assign to users
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, ParameterSetName = "Default")]
	[ValidateNotNullOrEmpty()]
	[string] $PolicyName,
	
	# Type of policy to assign to users
	[parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $true, ParameterSetName = "Default")] 
	[ValidateNotNullOrEmpty()]
	[ValidateSet("Archiving","Client","ClientVersion","Conferencing","ExternalAccess","HostedVoicemail","Location","Mobility","Pin","Presence","Voice")]
	[string] $PolicyType = "ExternalAccess",
	
	# Specifies install option, which creates a scheduled task for the script. It does NOT create the required Exchange receive connector.
	[parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $true, ParameterSetName = "Install")] 
	[switch] $Install
)
[string] $ScriptName = $MyInvocation.MyCommand.Name
[string] $ScriptPathAndName = $MyInvocation.MyCommand.Definition

Write-Verbose "Defining functions"
function Set-ModuleStatus { 
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")] 
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object		
		if (!(Get-Module -name "$name")) { 
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") { 
				Import-Module -Name "$name" 
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		} 
	} # end PROCESS
} # end function Set-ModuleStatus

function Remove-ScriptVariables {  
	[cmdletBinding(SupportsShouldProcess = $true)]
	param(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)] 
		[string] $path
	)
	$result = Get-Content $path |  
	ForEach { 
		if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		Remove-Variable ($v.replace("$","")) -EA 0
	}
} # end function Remove-ScriptVariables

function Install	{
	[cmdletBinding(SupportsShouldProcess = $true)]
	param()
	BEGIN	{	
		$error.clear()
		Write-Output "Creating scheduled task `"$ScriptName`"..."
		$TaskCreds = Get-Credential("$env:userdnsdomain\$env:username")		
		$TaskPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TaskCreds.Password))	
	}
	PROCESS	{
		schtasks /create /tn $ScriptName /tr "$env:windir\system32\windowspowershell\v1.0\powershell.exe -noprofile -command $ScriptPathAndName" /sc Hourly /MO 4 /ru $TaskCreds.UserName /rp $TaskPassword | Out-Null
		if (! $error){
			Write-Host "Installation complete!" -ForegroundColor green
		}else{
			Write-Host "Installation failed!" -ForegroundColor red
			exit
		}
	}
	END	{
		Remove-Variable TaskCreds
		Remove-Variable TaskPassword
	}	
} # end function Install

function Get-ADNestedGroupMembers{
	[cmdletBinding(SupportsShouldProcess = $true)]
	param ( 
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)] 
		[String] $Group
	) 
	Write-Verbose "Gathering group membership of $GroupDN"
	$Members = Get-ADGroupMember -Identity $Group
	# $Members | % {	
	$Members | ForEach-Object {	
		if ($_.ObjectClass -eq "group") {
			Write-Verbose "Getting nested group membership of $_.distinguishedName"
			Get-ADNestedGroupMembers -group $_.distinguishedName
		} else {
			$error.clear()
			[string] $name = $_.samaccountname
			Write-Verbose ""
			if ((Get-CsAdUser $name -ea 0).enabled){
				Write-Verbose "$name is enabled in Lync"
				[string] $cmd = "Grant-CS"+$PolicyType+"Policy -Identity "+$name+" -policyname `"$PolicyName`""
				Write-Verbose "Granting $PolicyType policy `"$PolicyName`""
				Invoke-Expression $cmd
				if (! $error){
					Write-Verbose "Successfully granted `"$PolicyName`" to $name"
				}else{
					Write-Verbose "Failed granting `"$PolicyName`" to $name: $error"
				}
			}else{
				Write-Verbose "$name is NOT enabled in Lync"
			}
			# return $_
		}
	}
} # end function Get-ADNestedGroupMembers

if ($install){
	Write-Verbose "Install mode"
	Install
	Exit
}

$error.clear()
Write-Verbose "Loading modules"

Set-ModuleStatus Lync
Set-ModuleStatus ActiveDirectory

Get-ADNestedGroupMembers -group $GroupDN

Remove-ScriptVariables $ScriptPathAndName
